@@ -8,6 +8,7 @@ use {
     crate::{block_cost_limits::*, execute_cost_table::ExecuteCostTable},
     log::*,
     solana_sdk::{pubkey::Pubkey, transaction::SanitizedTransaction},
+    std::collections::HashMap,
 };
 
 const MAX_WRITABLE_ACCOUNTS: usize = 256;
@@ -73,9 +74,28 @@ impl CostModel {
             .map(|(key, cost)| (key, cost))
             .chain(BUILT_IN_INSTRUCTION_COSTS.iter())
             .for_each(|(program_id, cost)| {
-                self.instruction_execution_cost_table
-                    .upsert(program_id, *cost);
+                match self
+                    .instruction_execution_cost_table
+                    .upsert(program_id, *cost)
+                {
+                    Some(c) => {
+                        debug!(
+                            "initiating cost table, instruction {:?} has cost {}",
+                            program_id, c
+                        );
+                    }
+                    None => {
+                        debug!(
+                            "initiating cost table, failed for instruction {:?}",
+                            program_id
+                        );
+                    }
+                }
             });
+        debug!(
+            "restored cost model instruction cost table from blockstore, current values: {:?}",
+            self.get_instruction_cost_table()
+        );
     }
 
     pub fn calculate_cost(&self, transaction: &SanitizedTransaction) -> TransactionCost {
@@ -90,20 +110,30 @@ impl CostModel {
         tx_cost
     }
 
-    // update-or-insert op is always successful. However the result of upsert, eg the aggregated
-    // value, requires additional calculation, which should only be envoked when needed.
-    pub fn upsert_instruction_cost(&mut self, program_key: &Pubkey, cost: u64) {
+    pub fn upsert_instruction_cost(
+        &mut self,
+        program_key: &Pubkey,
+        cost: u64,
+    ) -> Result<u64, &'static str> {
         self.instruction_execution_cost_table
             .upsert(program_key, cost);
+        match self.instruction_execution_cost_table.get_cost(program_key) {
+            Some(cost) => Ok(*cost),
+            None => Err("failed to upsert to ExecuteCostTable"),
+        }
+    }
+
+    pub fn get_instruction_cost_table(&self) -> &HashMap<Pubkey, u64> {
+        self.instruction_execution_cost_table.get_cost_table()
     }
 
     pub fn find_instruction_cost(&self, program_key: &Pubkey) -> u64 {
         match self.instruction_execution_cost_table.get_cost(program_key) {
-            Some(cost) => cost,
+            Some(cost) => *cost,
             None => {
-                let default_value = self.instruction_execution_cost_table.get_default();
+                let default_value = self.instruction_execution_cost_table.get_mode();
                 debug!(
-                    "instruction {:?} does not have aggregated cost, using default {}",
+                    "Program key {:?} does not have assigned cost, using mode {}",
                     program_key, default_value
                 );
                 default_value
@@ -181,7 +211,6 @@ mod tests {
             transaction::Transaction,
         },
         std::{
-            collections::HashMap,
             str::FromStr,
             sync::{Arc, RwLock},
             thread::{self, JoinHandle},
@@ -205,16 +234,18 @@ mod tests {
         let mut testee = CostModel::default();
 
         let known_key = Pubkey::from_str("known11111111111111111111111111111111111111").unwrap();
-        testee.upsert_instruction_cost(&known_key, 100);
+        testee.upsert_instruction_cost(&known_key, 100).unwrap();
         // find cost for known programs
         assert_eq!(100, testee.find_instruction_cost(&known_key));
 
-        testee.upsert_instruction_cost(&bpf_loader::id(), 1999);
+        testee
+            .upsert_instruction_cost(&bpf_loader::id(), 1999)
+            .unwrap();
         assert_eq!(1999, testee.find_instruction_cost(&bpf_loader::id()));
 
         // unknown program is assigned with default cost
         assert_eq!(
-            testee.instruction_execution_cost_table.get_default(),
+            testee.instruction_execution_cost_table.get_mode(),
             testee.find_instruction_cost(
                 &Pubkey::from_str("unknown111111111111111111111111111111111111").unwrap()
             )
@@ -232,7 +263,7 @@ mod tests {
         });
 
         test_key_and_cost.iter().for_each(|(key, cost)| {
-            testee.upsert_instruction_cost(key, *cost);
+            let _ = testee.upsert_instruction_cost(key, *cost).unwrap();
             info!("key {:?} cost {}", key, cost);
         });
 
@@ -267,7 +298,9 @@ mod tests {
         let expected_cost = 8;
 
         let mut testee = CostModel::default();
-        testee.upsert_instruction_cost(&system_program::id(), expected_cost);
+        testee
+            .upsert_instruction_cost(&system_program::id(), expected_cost)
+            .unwrap();
         assert_eq!(
             expected_cost,
             testee.get_transaction_cost(&simple_transaction)
@@ -295,7 +328,9 @@ mod tests {
         let expected_cost = program_cost * 2;
 
         let mut testee = CostModel::default();
-        testee.upsert_instruction_cost(&system_program::id(), program_cost);
+        testee
+            .upsert_instruction_cost(&system_program::id(), program_cost)
+            .unwrap();
         assert_eq!(expected_cost, testee.get_transaction_cost(&tx));
     }
 
@@ -327,7 +362,7 @@ mod tests {
         let result = testee.get_transaction_cost(&tx);
 
         // expected cost for two random/unknown program is
-        let expected_cost = testee.instruction_execution_cost_table.get_default() * 2;
+        let expected_cost = testee.instruction_execution_cost_table.get_mode() * 2;
         assert_eq!(expected_cost, result);
     }
 
@@ -371,12 +406,12 @@ mod tests {
         let mut cost_model = CostModel::default();
         // Using default cost for unknown instruction
         assert_eq!(
-            cost_model.instruction_execution_cost_table.get_default(),
+            cost_model.instruction_execution_cost_table.get_mode(),
             cost_model.find_instruction_cost(&key1)
         );
 
         // insert instruction cost to table
-        cost_model.upsert_instruction_cost(&key1, cost1);
+        assert!(cost_model.upsert_instruction_cost(&key1, cost1).is_ok());
 
         // now it is known insturction with known cost
         assert_eq!(cost1, cost_model.find_instruction_cost(&key1));
@@ -396,7 +431,9 @@ mod tests {
         let expected_execution_cost = 8;
 
         let mut cost_model = CostModel::default();
-        cost_model.upsert_instruction_cost(&system_program::id(), expected_execution_cost);
+        cost_model
+            .upsert_instruction_cost(&system_program::id(), expected_execution_cost)
+            .unwrap();
         let tx_cost = cost_model.calculate_cost(&tx);
         assert_eq!(expected_account_cost, tx_cost.write_lock_cost);
         assert_eq!(expected_execution_cost, tx_cost.execution_cost);
@@ -408,17 +445,16 @@ mod tests {
         let key1 = Pubkey::new_unique();
         let cost1 = 100;
         let cost2 = 200;
-        // updated_cost = (mean + 2*std) of [100, 200] => 120.899
-        let updated_cost = 121;
+        let updated_cost = (cost1 + cost2) / 2;
 
         let mut cost_model = CostModel::default();
 
         // insert instruction cost to table
-        cost_model.upsert_instruction_cost(&key1, cost1);
+        assert!(cost_model.upsert_instruction_cost(&key1, cost1).is_ok());
         assert_eq!(cost1, cost_model.find_instruction_cost(&key1));
 
         // update instruction cost
-        cost_model.upsert_instruction_cost(&key1, cost2);
+        assert!(cost_model.upsert_instruction_cost(&key1, cost2).is_ok());
         assert_eq!(updated_cost, cost_model.find_instruction_cost(&key1));
     }
 
@@ -460,8 +496,8 @@ mod tests {
                 if i == 5 {
                     thread::spawn(move || {
                         let mut cost_model = cost_model.write().unwrap();
-                        cost_model.upsert_instruction_cost(&prog1, cost1);
-                        cost_model.upsert_instruction_cost(&prog2, cost2);
+                        assert!(cost_model.upsert_instruction_cost(&prog1, cost1).is_ok());
+                        assert!(cost_model.upsert_instruction_cost(&prog2, cost2).is_ok());
                     })
                 } else {
                     thread::spawn(move || {
