@@ -274,7 +274,7 @@ mod tests {
             executors.clone(),
             &mut Vec::new(),
             Arc::new(FeatureSet::all_enabled()),
-            ComputeBudget::new(),
+            ComputeBudget::default(),
             &mut ExecuteTimings::default(),
             &sysvar_cache,
             Hash::default(),
@@ -304,7 +304,7 @@ mod tests {
             executors.clone(),
             &mut Vec::new(),
             Arc::new(FeatureSet::all_enabled()),
-            ComputeBudget::new(),
+            ComputeBudget::default(),
             &mut ExecuteTimings::default(),
             &sysvar_cache,
             Hash::default(),
@@ -338,7 +338,7 @@ mod tests {
             executors,
             &mut Vec::new(),
             Arc::new(FeatureSet::all_enabled()),
-            ComputeBudget::new(),
+            ComputeBudget::default(),
             &mut ExecuteTimings::default(),
             &sysvar_cache,
             Hash::default(),
@@ -484,7 +484,7 @@ mod tests {
             executors.clone(),
             &mut Vec::new(),
             Arc::new(FeatureSet::all_enabled()),
-            ComputeBudget::new(),
+            ComputeBudget::default(),
             &mut ExecuteTimings::default(),
             &sysvar_cache,
             Hash::default(),
@@ -518,7 +518,7 @@ mod tests {
             executors.clone(),
             &mut Vec::new(),
             Arc::new(FeatureSet::all_enabled()),
-            ComputeBudget::new(),
+            ComputeBudget::default(),
             &mut ExecuteTimings::default(),
             &sysvar_cache,
             Hash::default(),
@@ -549,7 +549,7 @@ mod tests {
             executors,
             &mut Vec::new(),
             Arc::new(FeatureSet::all_enabled()),
-            ComputeBudget::new(),
+            ComputeBudget::default(),
             &mut ExecuteTimings::default(),
             &sysvar_cache,
             Hash::default(),
@@ -607,7 +607,7 @@ mod tests {
             Rc::new(RefCell::new(Executors::default())),
             &mut Vec::new(),
             Arc::new(FeatureSet::all_enabled()),
-            ComputeBudget::new(),
+            ComputeBudget::default(),
             &mut ExecuteTimings::default(),
             &sysvar_cache,
             Hash::default(),
